/***************************************************************************
* 
* Copyright (c) 1998, Decision Systems Group
*
* Permission is hereby granted, free of charge, to any person obtaining
* a copy of this software and associated documentation files (the
* "Software"), to deal in the Software without restriction, including
* without limitation the rights to use, copy, modify, merge, publish,
* distribute, sublicense, and/or sell copies of the Software, and to
* permit persons to whom the Software is furnished to do so, subject to
* the following conditions:
*
* The above copyright notice and this permission notice shall be
* included in all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
* 
* ==========================================================================
* 
* FILE:
* 
* CU_Array.hh
* 
* AUTHOR: 
* 
* Stephan R.A. Deibel
* 
* CREATION DATE: 
* 
* 9.25.95
* 
* DESCRIPTION: 
* 
* Simple C++ array class implementation.
*
* NOTES:
*
*
* MODIFICATIONS: 
* --------------------------------------------------------------------------
* Date     Name      Description of modification
* --------------------------------------------------------------------------
* 
* 9/25/95  Stephan   Created
* 
*/

#ifndef CU_ARRAY_HH_ALREADY_INCLUDED
#define CU_ARRAY_HH_ALREADY_INCLUDED

#ifdef dos
#include "CUBasics.h"
#include "CUFail.h"
#include "CUDebug.h"
#else
#include "CU_Basics.h"
#include "CU_Failure.h"
#include "CU_Debug.h"
#endif


/**************************************************************************/
/* CLASSES */

class CUC_Array {

  public:

  CUC_Array(tUInt4 item_size);
  ~CUC_Array();

  tUInt4 GetNumItems() const;
  tUInt4 SetNumItems(tUInt4 n);

  void SetItem(tUInt4 n, void *new_item);
  void GetItem(tUInt4 n, void *item) const;
  void *GetItemPtr(tUInt4 n);

  tUInt4 Append(void *new_item);
  tUInt4 Prepend(void *new_item);
  tUInt4 Insert(tUInt4 n, void *new_item);
  void Remove(tUInt4 pos, tUInt4 num);

  CUC_Array *Copy() const;

  void SetGrowthIncr(tUInt4 n);

  protected:

  private:

  void *fArray;          /* The array */
  tUInt4 fNumItems;      /* Number of items in list */
  tUInt4 fItemSize;      /* Size of each item */
  tUInt4 fAllocated;     /* Allocated length (>= fNumItems * fItemSize) */
  tUInt4 fGrowthIncr;    /* Size to grow by when growing array */

  tUInt4 GrowMemory(tUInt4 num_items);
};


#endif /* CU_ARRAY_HH_ALREADY_INCLUDED */
