/***************************************************************************
* 
* Copyright (c) 1998, Decision Systems Group
*
* Permission is hereby granted, free of charge, to any person obtaining
* a copy of this software and associated documentation files (the
* "Software"), to deal in the Software without restriction, including
* without limitation the rights to use, copy, modify, merge, publish,
* distribute, sublicense, and/or sell copies of the Software, and to
* permit persons to whom the Software is furnished to do so, subject to
* the following conditions:
*
* The above copyright notice and this permission notice shall be
* included in all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
* 
* ==========================================================================
* 
* FILE: CU_Basics.h
* 
* AUTHOR: 
*
* Stephan R.A. Deibel
* YongJoon Lee
* Jonathan H. Traum
* Edward Pattison-Gordon
* 
* CREATION DATE: 
* 
* 8/1/93
*
* VERSION: 
* 
* 8/8/94
*
* DESCRIPTION: 
* 
* Basic support for cross-platform development.
* 
* NOTES: 
* 
* 
* MODIFICATIONS: 
* --------------------------------------------------------------------------
* Date     Name      Description of modification
* --------------------------------------------------------------------------
* 
* 8/1/93   Stephan   Initial implementation based on Jon's previous work
* 12/15/93 Stephan   Added constants for Max/Min of basic integer types
* 8/8/94   Stephan   Cleaned up mess in CUt_Text support functions, added
*                    missing comments, etc.  Made some changes in parameters
*                    of text functions as well for consistency.  Added
*                    ASSERTS.  Filled in skeletons for functions previously
*                    only in the header file ??!?!  What a mess!
*                    Also added support for CU_MallocSet().
* 8/18/94  YJLee     Added memmove and SET macros.
*
* 11/15/94 Karen     Added MS Windows definitions.
* 
* 12/8/94  YJLee     Changed MIN, MAX... macros to CU_MIN, CU_MAX etc.
*    
*/

#ifndef CU_BASICS_H_ALREADY_INCLUDED
#define CU_BASICS_H_ALREADY_INCLUDED

#include <errno.h>

/* Include runtime type information support code if using GCC 2.x */
/* instead of egcs */
#ifdef DOESNOTWORK /* gcc does not work ... oh well [SRAD] */
#ifdef GCC_2
#ifdef __cplusplus
#include <std/typeinfo.h>
#endif
#endif
#endif

#if defined(MS_WIN) && defined(MS_VC)
#ifdef WIN32
#define EXPORT_ONLY __declspec(dllexport)
#define EXPORT_PREFIX __declspec(dllexport)
#else
#define EXPORT_ONLY __export
#define EXPORT_PREFIX _export _far _pascal
#endif // !WIN32
#else
#define EXPORT_PREFIX
/*
#define hmemcpy memcpy
#define lstrcat strcat
#define lstrcmp strcmp
#define lstrcpy strcpy
#define lstrlet setlen
#define gmtime localtime
#define FAR
#define PASCAL            
#define EXPORT_ONLY
#define _huge 
typedef char *LPSTR;
typedef char *LPCSTR;  
typedef void *LPVOID;             
*/
#endif /* !MS_WIN */

#include <stdlib.h>

#ifdef __cplusplus
extern "C" {
#endif

#ifdef __BORLANDC__
#pragma warn -sig
#endif


/**************************************************************************/
/* CONSTANTS */

#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif

#ifndef NULL
#define NULL 0L
#endif

#define kMinUInt1 0
#define kMaxUInt1 255U
#define kMinUInt2 0
#define kMaxUInt2 65535U
#define kMinUInt4 0
#define kMaxUInt4 4294967295UL
#define kMinUInt8 0
#define kMaxUInt8 18446730000000000000000000ULL /* estimate */

#define kMinInt1 -127
#define kMaxInt1 127
#define kMinInt2 -32767
#define kMaxInt2 32767
#define kMinInt4 -2147483647
#define kMaxInt4 2147483647L
#define kMinInt8 -9223371000000000000000000LL /* estimate */
#define kMaxInt8 9223371000000000000000000LL /* estimate */

#define	CUk_PI (tFloat) 3.14159265358979323846

/* Big / Little endian determination */
#ifdef macintosh
#  define CUk_BigEndian
#endif /* defined macintosh */
#ifdef MS_WIN
#  define CUk_LittleEndian
#endif /* defined MS_WIN */
#ifdef unix
#  if defined (linux)
#    include <endian.h>
#    if BYTE_ORDER == LITTLE_ENDIAN
#      define CUk_LittleEndian
#    else
#      define CUk_BigEndian
#    endif
#  endif /* defined linux */

#  if defined (hpux)
#    define CUk_BigEndian
#  endif /* defined hpux */

#  if defined (sunos4)
#    define CUk_BigEndian
#  endif

#endif /* defined unix */

#if !defined(CUk_BigEndian) && !defined(CUk_LittleEndian)
#  error Byte order for this system type is not defined for this OS 
#endif

// Missing linux constants
#ifdef linux
#ifndef ENOERR
#define ENOERR 0
#endif
#endif


/**************************************************************************/
/* TYPES */

#ifdef THINK_C
typedef char tInt1;
typedef short int tInt2;
typedef long int tInt4;
typedef unsigned char tUInt1;
typedef unsigned short int tUInt2;
typedef unsigned long int tUInt4;
typedef float tFloat;
typedef float tFloat4;
typedef double tFloat8;
typedef long double tFloat16;
typedef char tOctet;
typedef char tInt8 [8];
typedef char tUInt8 [8];

#elif defined(symantec)
typedef char tInt1;
typedef short int tInt2;
typedef long int tInt4;
typedef unsigned char tUInt1;
typedef unsigned short int tUInt2;
typedef unsigned long int tUInt4;
typedef float tFloat;
typedef float tFloat4;
typedef double tFloat8;
typedef long double tFloat16;
typedef char tOctet;
typedef char tInt8 [8];
typedef char tUInt8 [8];

#elif defined(__GNUC__)
typedef char tInt1;
typedef short int tInt2;
typedef long int tInt4;
typedef unsigned char tUInt1;
typedef unsigned short int tUInt2;
typedef unsigned long int tUInt4;
typedef float tFloat;
typedef float tFloat4;
typedef double tFloat8;
typedef long double tFloat16;
typedef char tOctet;
typedef long long tInt8;
typedef unsigned long long tUInt8;

#elif defined(__BORLANDC__)
typedef char tInt1;
typedef short int tInt2;
typedef long int tInt4;
typedef unsigned char tUInt1;
typedef unsigned short int tUInt2;
typedef unsigned long int tUInt4;
typedef float tFloat;
typedef float tFloat4;
typedef double tFloat8;
typedef long double tFloat16;
typedef char tOctet;
typedef char tInt8 [8];
typedef char tUInt8 [8];

#elif defined(__MWERKS__)
typedef char tInt1;
typedef short int tInt2;
typedef long int tInt4;
typedef unsigned char tUInt1;
typedef unsigned short int tUInt2;
typedef unsigned long int tUInt4;
typedef float tFloat;
typedef float tFloat4;
typedef double tFloat8;
typedef long double tFloat16;
typedef char tOctet;
#ifdef WIN32
	typedef long long tInt8;
	typedef unsigned long long tUInt8;
#else // macintosh
	typedef char tInt8 [8];
	typedef char tUInt8 [8];
#endif // WIN32

#elif defined(MS_VC)
typedef char tInt1;
typedef short int tInt2;
typedef long int tInt4;
typedef unsigned char tUInt1;
typedef unsigned short int tUInt2;
typedef unsigned long int tUInt4;
typedef float tFloat;
typedef float tFloat4;
typedef double tFloat8;
typedef long double tFloat16;
typedef unsigned char tOctet;
typedef __int64 tInt8;
typedef unsigned __int64 tUInt8;

#else
#error Unknown compiler environment
#endif /* Compilers */

typedef tUInt1 tBoolean;

typedef struct {
  tUInt4 HostID;
  tUInt4 ProcessID;
  tUInt4 Time;
  tUInt4 ID;
} CUt_UniqueID;

/*
* Error type -- value must be one of the constants defined above
*/

typedef tUInt2 tError;


/**************************************************************************/
/* MACROS */

/* Numerical */
#define CU_MIN(a,b) (((a) < (b)) ? (a) : (b))
#define CU_MAX(a,b) (((a) > (b)) ? (a) : (b))
#define CU_ROUND(x) ((x > 0) ? (tInt4)(x+0.5) : ((tInt4)(x-0.5)))
#define CU_ROUND_UP(x, y) (((x) / (y)) + ((x) % (y) ? 1 : 0))
#define CU_ABS(x) (((x) < 0) ? (-(x)) : (x))

#ifdef macintosh
/* Missing commonly used BSD routines */
#undef index
#undef rindex
#define index(a, b)  strchr(a, b)
#define rindex(a, b) strrchr(a, b)

#undef bzero
#undef bcopy
#undef bcmp
#undef bfill
#define bzero(from, len)     memset(from, 0, len)
#define bcopy(from, to, len) memcpy(to, from, len)
#define bcmp(s1, s2, len)    memcmp(s1, s2, len)
#define bfill(from, len, x)  memset(from, x, len)

#undef isascii
#define isascii(ch) ((unsigned char) ch >=0 &&  (unsigned char ) ch <= 127) ? 1 : 0

#endif /* macintosh */

#ifdef __GNUC__

#ifdef sunos4
#ifndef memmove
#define memmove(x,y,z) bcopy((char*)(x), (char *)(y), (z))
#endif
#endif

/* for some reason not defined in unix <stdio.h> */
#ifndef SEEK_SET
#define SEEK_SET 0
#endif

#ifndef SEEK_CUR
#define SEEK_CUR 1
#endif

#ifndef SEEK_END
#define SEEK_END 2
#endif

#endif /* __GNUC__ */

#if defined(WIN32) && !defined(__GNUC__)

/* Missing commonly used BSD routines */
#define index(a, b)  strchr(a, b)
#define rindex(a, b) strrchr(a, b)

#ifndef bzero
#define bzero(from, len)     memset(from, 0, len)
#endif
#ifndef bcopy
#define bcopy(from, to, len) memcpy(to, from, len)
#endif
#ifndef bcmp
#define bcmp(s1, s2, len)    memcmp(s1, s2, len)
#endif
#ifndef bfill
#define bfill(from, len, x)  memset(from, x, len)
#endif

#define isatty(x) 0 
  
/* Work-arounds for routines renamed in windows */
#if defined(__MWERKS__)
#define strdup _strdup
#endif /* __MWERKS__ */
  
#endif /* WIN32 but not __GNUC__ */ 
  

/* Misc */
#ifdef macintosh
#define CUk_FileSeperator ":"
#elif defined(unix)
#define CUk_FileSeperator "/"
#elif defined(dos) || defined(MS_WIN)
#define CUk_FileSeperator "\\"
#else
#error "Platform identifier undefined"
#endif


/**************************************************************************/
/* PROTOTYPES */

/*
* Initialize CU 
*/

EXPORT_PREFIX tError CU_Initialize();

/*
* Memory allocation/deallocation functions with debug support
*/ 

EXPORT_PREFIX void * CU_Malloc(tUInt4 mem_size);
EXPORT_PREFIX void * CU_Calloc(tUInt4 num, tUInt4 size);
EXPORT_PREFIX void * CU_Realloc(void *ptr, tUInt4 size);
EXPORT_PREFIX void * CU_MallocSet(tUInt4 mem_size, tUInt1 fill);
EXPORT_PREFIX void CU_Free(void * mem);

/*
* Memory Utility Functions 
*/ 

EXPORT_PREFIX void * CU_MemCpy( void *dest, const void *src, tUInt4 count );
EXPORT_PREFIX void * CU_MemMove( void *dest, const void *src, tUInt4 count );
EXPORT_PREFIX tInt4 CU_MemCmp( const void *buf1, const void *buf2, tUInt4 count );
EXPORT_PREFIX void * CU_MemSet( void *dest, char c, tUInt4 count );

EXPORT_PREFIX tUInt4 CU_MaxMem(void);

EXPORT_PREFIX void CU_InvertMem(void* buff, tUInt4 buffsize);


/*
* Unique ID Functions
*/ 

EXPORT_PREFIX tBoolean CU_CreateUniqueID(CUt_UniqueID *ID);
EXPORT_PREFIX void CU_InitUniqueID(CUt_UniqueID *ID);
EXPORT_PREFIX tBoolean CU_UniqueIDEqual(const CUt_UniqueID *id1, 
					const CUt_UniqueID *id2);


/*
* printf() is not defined in MS Windows 
*/

#ifdef MS_WIN
#if 0
#if defined( MS_VC ) && defined( MS_NO_PRINTF )
#define printf() ((void *)0)
#endif /* Visual C++ DLL problem */
#endif


#endif /* MS_WIN */


#ifdef __cplusplus
} /* extern "C" */
#endif 

#endif /* CU_BASICS_H_ALREADY_INCLUDED */
