/***************************************************************************
* 
* Copyright (c) 1998, Decision Systems Group
*
* Permission is hereby granted, free of charge, to any person obtaining
* a copy of this software and associated documentation files (the
* "Software"), to deal in the Software without restriction, including
* without limitation the rights to use, copy, modify, merge, publish,
* distribute, sublicense, and/or sell copies of the Software, and to
* permit persons to whom the Software is furnished to do so, subject to
* the following conditions:
*
* The above copyright notice and this permission notice shall be
* included in all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
* 
* ==========================================================================
* 
* FILE: CU_CStream.h
* 
* AUTHOR: 
* 
* Jaime Requena
* 
* CREATION DATE: 
* 
* May 1997
*
* DESCRIPTION: 
* 
* Simple streams implementation, with slightly better error checking.
* 
* NOTES: 
* 
* 
* MODIFICATIONS: 
* --------------------------------------------------------------------------
* Date     Name      Description of modification
* --------------------------------------------------------------------------
* 
* 
*/

#ifndef CU_CSTREAM_H_ALREADY_INCLUDED
#define CU_CSTREAM_H_ALREADY_INCLUDED


/**************************************************************************/
/* INCLUDES */

#include "CU_Basics.h"
#include "CU_Failure.h"
#include <stdio.h>
#include "CU_Text.h"

 #ifdef __cplusplus
extern "C" {
#endif

typedef struct {
  tError error;
  char* message;
} CUt_Environment;

typedef struct {
  char *name;
  FILE *file;
} CUt_Stream;

void CU_InitStream(CUt_Stream* stream);
void CU_OpenStream( CUt_Environment *ev, CUt_Stream* stream, const char *filename );
void CU_CloseStream(CUt_Stream *stream);
tInt2 CU_GetStream(const CUt_Stream *stream);
tUInt4 CU_ReadStream(const CUt_Stream *stream, void *dest,tUInt4 n);
tError CU_SeekgStream(const CUt_Stream *stream, tUInt4 pos);
tUInt4 CU_TellgStream(const CUt_Stream *stream);
void CU_StreamGetString(CUt_Environment* ev, const CUt_Stream *stream, 
			char* string, tUInt2 length);
void CU_StreamWriteString(CUt_Environment* ev, const CUt_Stream *stream, const char* string);

#ifdef __cplusplus
}
#endif


#endif /* CU_CSTREAM_H_ALREADY_INCLUDED */
