/***************************************************************************
* 
* Copyright (c) 1998, Decision Systems Group
*
* Permission is hereby granted, free of charge, to any person obtaining
* a copy of this software and associated documentation files (the
* "Software"), to deal in the Software without restriction, including
* without limitation the rights to use, copy, modify, merge, publish,
* distribute, sublicense, and/or sell copies of the Software, and to
* permit persons to whom the Software is furnished to do so, subject to
* the following conditions:
*
* The above copyright notice and this permission notice shall be
* included in all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
* 
* ==========================================================================
* 
* FILE: CU_DStream.hh
* 
* AUTHOR: 
*
* Ed Pattison-Gordon
* John Ehresman
* Stephan Deibel
* 
* CREATION DATE: 
* 
* July 1995
*
* DESCRIPTION: 
*
* A fairly simple dynamic string class
* 
* NOTES: 
* 
* MODIFICATIONS: 
* --------------------------------------------------------------------------
* Date     Name      Description of modification
* --------------------------------------------------------------------------
* 
* $Log: CU_DString.hh,v $
* Revision 1.2  1998/07/15 05:20:14  sdeibel
* Changed all copyright notices to match MIT license release plus misc
* prep for 0.8.3 release
*
// Revision 1.1.1.1  1997/10/24  16:48:44  sdeibel
// Converting to CVS
//
* Revision 1.3  1996/01/24 18:59:29  sdeibel
* Reimplemented using new CU_Text.*
*
*
*/

#ifndef CU_DSTRING_HH_ALREADY_INCLUDED
#define CU_DSTRING_HH_ALREADY_INCLUDED


/**************************************************************************/
/* INCLUDES */

#include "CU_Basics.h"
#include "CU_Failure.h"
#include "CU_Text.h"


/**************************************************************************/
/* CONSTANTS */

#define CUk_DefDStringBlockSize 32


/**************************************************************************/
/* CLASSES */

class CUC_DString {
public:

  // Methods
  CUC_DString ();
  ~CUC_DString ();
  
  void Clear()
    { CU_TXClear(&fText); };

  const char *GetString () const
    { return CU_TXGetCStringPtr(&fText); };
  
  char *TakeString ();
  
  tUInt4 GetLength () const
    { return CU_TXGetLength(&fText); };
  
  tError SetString(const char *the_string, tUInt4 len)
    { return CU_TXSetText(&fText, the_string, len); };

  tError AppendChar(const char the_character)
    { return CU_TXAppendChar(&fText, the_character); };
  
  tError AppendString(const char *the_string, tUInt4 len)
    { return CU_TXAppendText(&fText, the_string, len); };

  tError AppendCString(const char *str)
    { return CU_TXAppendCString(&fText, str); };
  
  tError AppendDString(const CUC_DString & str)
    { return CU_TXAppendText(&fText, str.GetString(), str.GetLength()); };

private:
  CUt_Text fText;

  tError ExpandBuffer(tUInt4 needed_len)
    { return CU_TXExpandBuffer(&fText, needed_len); };

}; /* CUC_DString */



#endif /* CU_DSTRING_HH_ALREADY_INCLUDED */
