/***************************************************************************
* 
* Copyright (c) 1998, Decision Systems Group
*
* Permission is hereby granted, free of charge, to any person obtaining
* a copy of this software and associated documentation files (the
* "Software"), to deal in the Software without restriction, including
* without limitation the rights to use, copy, modify, merge, publish,
* distribute, sublicense, and/or sell copies of the Software, and to
* permit persons to whom the Software is furnished to do so, subject to
* the following conditions:
*
* The above copyright notice and this permission notice shall be
* included in all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
* 
* ==========================================================================
* 
* FILE: 
*
* CU_Failure.h
* 
* AUTHOR: 
* 
* Stephan R.A. Deibel
*
* CREATION DATE: 
* 
* 8/15/93
*
* VERSION: 
* 
* 8/8/94
*
* DESCRIPTION: 
* 
* Failure types and utilities for failure identification and handling.
* 
* NOTES: 
* 
* This file includes failure codes for acceptible resource-related
* runtime failure that programs should handle in a graceful manner.
* Note that it does not include codes for runtime errors that are
* the result of incorrect program code.
*
* It is very important to distinguish acceptible failure from program
* error.  Program error should be obvious and fatal and there should
* by no non-DEBUG logic in a program that deals with the occurence
* of such errors.  Acceptible failure (e.g., out of memory, disk
* full, etc) does need to be dealt with by non-DEBUG logic, which
* should usually be kept to minimum due to the difficultly in fully
* debugging such rarely executed code.
*
* See the Decision Systems Group's Development Guidelines document
* for additional information on writing good failure and program
* error detection code.
*
* TO DO:
*
* Must remove bogus error types from here.  See those marked below.
* 
* MODIFICATIONS: 
* --------------------------------------------------------------------------
* Date     Name      Description of modification
* --------------------------------------------------------------------------
* 
* 8/15/93  Stephan   Created.
* 8/8/94   Stephan   Filled in comments, etc.  Added #include for CU_Basics.h.
* 10/11/94 Stephan   Changed file name to CU_Failure.c / added comments.
*
*/

#ifndef CU_FAILURE_H_ALREADY_INCLUDED
#define CU_FAILURE_H_ALREADY_INCLUDED

#ifdef dos
#include "CUBasics.h"
#else
#include "CU_Basics.h"
#endif /* dos */

#ifdef __cplusplus
extern "C" {
#endif


/**************************************************************************/
/* CONSTANTS */

#define kNoError 0
#define kOutOfMemory 1
#define kBadParam 2
  
/* File I/O Errors */
#define kFileIOError 100
#define kDirFullError 101
#define kDiskFullError 102
#define kVolumeError 103
#define kEOFError 105
#define kFileNotFound 106

/* Generic resource shortage errors */
#define kResourceNotFound 200
#define kResourceUnreachable 201
#define kResourceShortage 202

/* Implementation-dependent errors; range 500-999 */
#define kSpecialError0 500
#define kSpecialError1 501
#define kSpecialError2 502
#define kSpecialError3 503
#define kSpecialError4 504
#define kSpecialError5 505
#define kSpecialError6 506
#define kSpecialError7 507
#define kSpecialError8 508
#define kSpecialError9 509
/* etc. */

/* Other errors */
#define kInitFailure 32766
#define kUnknownError 32767


#ifdef __cplusplus
} /* extern "C" */
#endif 

#endif /* CU_FAILURE_H_ALREADY_INCLUDED */
