/***************************************************************************
* 
* Copyright (c) 1998, Decision Systems Group
* 
* Permission is hereby granted, free of charge, to any person obtaining
* a copy of this software and associated documentation files (the
* "Software"), to deal in the Software without restriction, including
* without limitation the rights to use, copy, modify, merge, publish,
* distribute, sublicense, and/or sell copies of the Software, and to
* permit persons to whom the Software is furnished to do so, subject to
* the following conditions:
* 
* The above copyright notice and this permission notice shall be
* included in all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
* 
* ==========================================================================
* 
* FILE:
* 
* CU_File.hh
* 
* AUTHOR: 
* 
* Jaime Requena
* Stephan R.A. Deibel
* 
* CREATION DATE: 
* 
* 9.02.96
* 
* DESCRIPTION: 
* 
* File and file name manipulation utilities
*
* NOTES:
*
*
* MODIFICATIONS: 
* 
* $Log: CU_File.hh,v $
* Revision 1.9  1998/12/04 22:44:34  bsloan
* CreateWindow conflict, GetFileNamesInDir fix, PC gs_server now works
*
* Revision 1.8  1998/12/01 23:18:49  bsloan
* Port to MetroWerks CodeWarrior Pro4 for Windows
*
* Revision 1.7  1998/07/06 15:44:33  jehresma
* Defined CUt_StringSeq and support functions.  Defined CU_GetFilenamesInDir
* and CU_JoinPathname.
*
* Revision 1.6  1998/03/17 22:01:35  sdeibel
* Misc params made const
*
* Revision 1.5  1998/03/17 20:59:01  jehresma
* Port to PC
*
* Revision 1.4  1998/03/17 19:27:14  sdeibel
* Relative/full path utils now work on UNIX
*
* Revision 1.3  1998/03/17 00:03:46  sdeibel
* Implemented relative path utils; not yet tested
*
* 
*/

#ifndef CU_FILE_ALREADY_INCLUDED
#define CU_FILE_ALREADY_INCLUDED

#ifndef WIN32
#include <unistd.h>
#endif /* WIN32 */
#include <stdio.h>
#include <time.h>
#include "CU_Basics.h"
#include "CU_Failure.h"
#include "CU_Sequence.h"


/****************************************************************************
* CONSTANTS */

#ifdef macintosh
#define	CUk_FilePathSeperator  ':'
#endif // macintosh

#if defined(dos) || defined(WIN32)
#define	CUk_FilePathSeperator  '\\'
#endif // dos or WIN32

#ifdef unix
#define	CUk_FilePathSeperator  '/'
#endif // unix

/* Indicator constants for relative path encoding */
#define CUk_RelPathIndicator 0x01
#define CUk_StripEntryIndicator 0x02


/****************************************************************************
* TYPES */

// Entry in sequence returned by CU_GetFilenamesInDir
typedef struct {
	char* name; // name of file or subdirectory
	tBoolean is_directory; // TRUE iff this is a subdirectory
} CUt_FileName;

typedef CU_DEFINE_SEQ(CUt_FileName) CUt_FileNameSeq;


/****************************************************************************
* PROTOTYPES */

// Create/open/close/remove files
tError CU_CreateFile(const char* newname);
tError CU_RemoveFile(const char* filename);

// File copy/move/rename
tError CU_CopyFile(const char* source, const char* dest);
tError CU_CopyFile(const char* source, FILE* dest);
tError CU_CopyFile(FILE* source, const char* dest);
tError CU_ExchangeFiles(const char* file_1, const char* file_2);
tError CU_ExchangeFiles(const char* file_1, FILE* file_2);
//tError CU_Exchange_Files(const char* file_1, const char* file_2);
//tError CU_Exchange_Files(const char* file_1, FILE* file_2);
tError CU_FileMove(const char* old_full_path, const char* new_file_path);
tError CU_RenameFile(const char* old_full_path, const char* newname);

// File existence/temp file utils
tBoolean CU_FileExists(const char* filename);
FILE* CU_TempFile();
FILE* CU_TempCopy (const char* source_file);

// File path/name manipulation utilities
char* CU_GetFileName (const char* full_path);
char* CU_GetFilePath (const char* full_path);
char* CU_GetUniqueFileName(const char* name_prefix);
char* CU_RemoveVolName(char *full_path);
char* CU_RemoveFileName(char *full_path);
void CU_FullToRelativePath(char *convert_path, const char *relative_to);
void CU_RelativeToFullPath(char *convert_path, const char *relative_to);
char *CU_RelativeToAscii(const char *relative);
char *CU_AsciiToRelative(const char *ascii);

CUt_FileNameSeq CU_GetFilenamesInDir(tError* err, 
																			const char* dir_name);
char* CU_JoinPathname(const char* dir_name, const char* file_name);

// These can't reasonably be cross-platform; should be removed [SRAD]
#if 0 // UNUSED
char* FullPath_From_Volume_ID_Directory_ID_And_Name (int volume_ID, long directory_ID, const char* file_name, tError* error_code);
char* FullPath_From_Default_Directory_And_Name (const char* file_name, tError* error_code);
#endif // UNUSED

#endif /* !CU_FILE_ALREADY_INCLUDED */


