/***************************************************************************
* 
* Copyright (c) 1998, Decision Systems Group
*
* Permission is hereby granted, free of charge, to any person obtaining
* a copy of this software and associated documentation files (the
* "Software"), to deal in the Software without restriction, including
* without limitation the rights to use, copy, modify, merge, publish,
* distribute, sublicense, and/or sell copies of the Software, and to
* permit persons to whom the Software is furnished to do so, subject to
* the following conditions:
*
* The above copyright notice and this permission notice shall be
* included in all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
* 
* ==========================================================================
* 
* FILE: CU_Hash.prv.h
* 
* AUTHOR: 
*
* YongJoon Lee
* John Ehresman
* Based on work of John K. Ousterhaut
* 
* CREATION DATE: 
*
* 95.9.22
*
* DESCRIPTION: 
*
* Public header file for hash table implementation.
* 
* NOTES: 
*
* Based on internal tcl routines v. 7.4b2.  The original copyright is 
* as follows:
*
*   Copyright (c) 1987-1994 The Regents of the University of California.
*   Copyright (c) 1994 Sun Microsystems, Inc.
*
*   See the file "tcl_license.terms" for information on usage and 
*   redistribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.
*
* Names have been modified to conform to the Arachne standard.
* 
* 
* MODIFICATIONS: 
* --------------------------------------------------------------------------
* Date     Name      Description of modification
* --------------------------------------------------------------------------
* 
* $Log: CU_Hash.prv.h,v $
* Revision 1.2  1998/07/15 05:20:15  sdeibel
* Changed all copyright notices to match MIT license release plus misc
* prep for 0.8.3 release
*
 * Revision 1.1.1.1  1997/10/24  16:48:44  sdeibel
 * Converting to CVS
 *
* Revision 1.4  1996/12/09 19:23:06  sdeibel
* Misc clean up
*
* 
*/

#ifndef CU_HASH_PRV_H_ALREADY_INCLUDED
#define CU_HASH_PRV_H_ALREADY_INCLUDED

#ifdef __cplusplus
extern "C" {
#endif


/**************************************************************************/
/* CONSTANTS */

/* When there are this many entries per bucket, on average, rebuild
 * the hash table to make it larger */
#define REBUILD_MULTIPLIER	3


/**************************************************************************/
/* MACROS */

/*
 * The following macro takes a preliminary integer hash value and
 * produces an index into a hash tables bucket list.  The idea is
 * to make it so that preliminary values that are arbitrarily similar
 * will end up in different buckets.  The hash function was taken
 * from a random-number generator.
 */

#define RANDOM_INDEX(tablePtr, i) \
    (((((long) (i))*1103515245) >> (tablePtr)->downShift) & (tablePtr)->mask)


/**************************************************************************/
/* PROTOTYPES */


static CUt_HashEntry *CU_ArrayFind(const CUt_HashTable *tablePtr, const void *key );
static CUt_HashEntry *CU_ArrayCreate( CUt_HashTable *tablePtr, const void *key, 
                            tInt4 *newPtr );
static CUt_HashEntry *CU_BogusFind(const CUt_HashTable *tablePtr, const void *key );
static CUt_HashEntry *CU_BogusCreate( CUt_HashTable *tablePtr, const void *key,
                            tInt4 *newPtr );
static tUInt4         CU_HashString(const char *string );
static void           CU_RebuildTable( CUt_HashTable *tablePtr );
static CUt_HashEntry *CU_StringFind(const CUt_HashTable *tablePtr, const void *key );
static CUt_HashEntry *CU_StringCreate(CUt_HashTable *tablePtr, const void *key,
                             tInt4 *newPtr );
#ifdef OLD
static CUt_HashEntry *CU_OneWordFind(const CUt_HashTable *tablePtr, const void *key);
static CUt_HashEntry *CU_OneWordCreate( CUt_HashTable *tablePtr, const void *key, 
                              tInt4 *newPtr );
#endif

#ifdef __cplusplus
} /* extern "C" */
#endif 

#endif /* CU_HASH_PRV_H_ALREADY_INCLUDED */
