/***************************************************************************
* 
* Copyright (c) 1998, Decision Systems Group
*
* Permission is hereby granted, free of charge, to any person obtaining
* a copy of this software and associated documentation files (the
* "Software"), to deal in the Software without restriction, including
* without limitation the rights to use, copy, modify, merge, publish,
* distribute, sublicense, and/or sell copies of the Software, and to
* permit persons to whom the Software is furnished to do so, subject to
* the following conditions:
*
* The above copyright notice and this permission notice shall be
* included in all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
* 
* ==========================================================================
* 
* FILE:
* 
* CU_Sequence.h
* 
* AUTHOR: 
* 
* Stephan R.A. Deibel
* YongJoon Lee
* John Ehresman
* 
* CREATION DATE: 
* 
* Oct 1996
* 
* DESCRIPTION: 
* 
* Sequence type support:  Compatible with CORBA sequences.
*
* NOTES:
*
*
* MODIFICATIONS: 
*
* $Log: CU_Sequence.h,v $
* Revision 1.3  1998/07/15 05:20:15  sdeibel
* Changed all copyright notices to match MIT license release plus misc
* prep for 0.8.3 release
*
* Revision 1.2  1998/07/06 15:44:34  jehresma
* Defined CUt_StringSeq and support functions.  Defined CU_GetFilenamesInDir
* and CU_JoinPathname.
*
 * Revision 1.1.1.1  1997/10/24  16:48:44  sdeibel
 * Converting to CVS
 *
* Revision 1.10  1997/04/07 12:08:41  jehresma
* Defined macro for sequence creation
*
* Revision 1.9  1997/03/03 23:40:53  sdeibel
* Added CU_SeqAppendArray
*
# Revision 1.8  1997/02/11  21:21:09  sdeibel
# Changed Remove not to have return error : Never fails!
#
# Revision 1.7  1996/12/17  17:47:16  sdeibel
# Added SeqRemove
#
# Revision 1.6  1996/11/25  20:03:22  sdeibel
# Added insert and sorting functions
#
# Revision 1.5  1996/11/11  22:54:58  sdeibel
# Re-added SeqPrepend()
#
# Revision 1.4  1996/10/24  21:23:40  jehresma
# Converted to C file
#
# Revision 1.2  1996/10/10  18:27:05  sdeibel
# Misc
#
# Revision 1.1  1996/10/10  18:10:49  sdeibel
# Initial revision
#
*
*/

#ifndef CU_SEQUENCE_H_ALREADY_INCLUDED
#define CU_SEQUENCE_H_ALREADY_INCLUDED

#include "CU_Basics.h"
#include "CU_Failure.h"
#include "CU_Debug.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/**************************************************************************/
/* TYPES */

/* All sequences must have this layout; usually, user will define */
/* types where the _buffer field is of a specific pointer type */
#define CU_DEFINE_SEQ(type) \
  struct { \
    tUInt4 _maximum; \
    tUInt4 _length; \
    type *_buffer; \
  }

typedef CU_DEFINE_SEQ(void) CUt_GenericSeq;    

/* Common sequence types */
typedef CU_DEFINE_SEQ(char*) CUt_StringSeq;

/* Ordering function for sorted insertion */
typedef tInt4 (*CUt_SeqSortFct) (void *insert_key, void *seq_entry);


/**************************************************************************/
/* PROTOTYPES */

/*
* C language sequence support
*/

void EXPORT_PREFIX CU_SeqInit(void* seq);
EXPORT_PREFIX void * CU_SeqCreate(void);
void EXPORT_PREFIX CU_SeqClear(void* seq);
void EXPORT_PREFIX CU_SeqDestroy(void* seq);
tError EXPORT_PREFIX CU_SeqExpand(void* seq, tUInt4 new_max, 
				  size_t el_size);
tError EXPORT_PREFIX CU_SeqSet(void* seq, tUInt4 index, void* data, 
			       size_t el_size);
tError EXPORT_PREFIX CU_SeqInsert(void * seq, tUInt4 index, void * data, 
				  size_t el_size);
void EXPORT_PREFIX CU_SeqRemove(void * seq, tUInt4 index, size_t el_size);
tError EXPORT_PREFIX CU_SeqInsertSorted(void *seq, void *data, size_t el_size,
					void *sort_key, 
					CUt_SeqSortFct sort_fct,
					tUInt4 *pos);
tBoolean EXPORT_PREFIX CU_SeqFindSorted(void *seq, size_t el_size, 
					void *sort_key,
					CUt_SeqSortFct sort_fct, 
					tUInt4 *pos);
tError EXPORT_PREFIX CU_SeqAppend(void* seq, void* data, size_t el_size);
tError EXPORT_PREFIX CU_SeqAppendArray(void* seq, tUInt4 num, void* data, 
				       size_t el_size);
tError EXPORT_PREFIX CU_SeqAppendSeq(void* dest_seq, void* src_seq, 
				     size_t el_size);
tError EXPORT_PREFIX CU_SeqCopy(const void* src_seq, void* dest_seq, 
				size_t el_size);
tError EXPORT_PREFIX CU_SeqPrepend(void * seq, void * data, size_t el_size);

void CU_StringSeqInit(CUt_StringSeq* seq);
void CU_StringSeqClear(CUt_StringSeq* seq);
tError CU_StringSeqAppendItem(CUt_StringSeq* seq, const char* str);

#ifdef __cplusplus
} /* ends extern "C" { */
#endif /* __cplusplus */

#endif /* CU_SEQUENCE_H_ALREADY_INCLUDED */

