/***************************************************************************
* 
* Copyright (c) 1998, Decision Systems Group
*
* Permission is hereby granted, free of charge, to any person obtaining
* a copy of this software and associated documentation files (the
* "Software"), to deal in the Software without restriction, including
* without limitation the rights to use, copy, modify, merge, publish,
* distribute, sublicense, and/or sell copies of the Software, and to
* permit persons to whom the Software is furnished to do so, subject to
* the following conditions:
*
* The above copyright notice and this permission notice shall be
* included in all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
* 
* ==========================================================================
* 
* FILE: CU_Text.h
* 
* AUTHOR: 
*
* Ed Pattison-Gordon
* John Ehresman
* Stephan Deibel
*
* CREATION DATE: 
*
* Jan 1996, from previous work
* 
* DESCRIPTION: 
*
* Header file for simple dynamic text / string support
* 
* NOTES: 
* 
* MODIFICATIONS: 
* --------------------------------------------------------------------------
* Date     Name      Description of modification
* --------------------------------------------------------------------------
* 
* $Log: CU_Text.h,v $
* Revision 1.2  1998/07/15 05:20:16  sdeibel
* Changed all copyright notices to match MIT license release plus misc
* prep for 0.8.3 release
*
 * Revision 1.1.1.1  1997/10/24  16:48:44  sdeibel
 * Converting to CVS
 *
* Revision 1.14  1997/09/16 14:54:05  jrequena
* Added type CUt_TextSeq and function CU_TXClearSequence
*
* Revision 1.13  1997/08/12  19:17:32  jrequena
* added CU_StripString
*
* Revision 1.12  1997/06/10  21:28:29  jehresma
* Implementation of CU_StrNCaseCmp
*
* Revision 1.11  1997/03/25 19:14:11  sdeibel
* Added CU_StrCharDelete
*
* Revision 1.10  1996/10/08  21:41:42  jehresma
* Added functions to insert and delete ranges
*
* Revision 1.9  1996/10/03  17:39:43  jehresma
* Added CU_TXUseTextPtr function
*
* Revision 1.8  1996/09/18  20:27:17  sdeibel
* Added conversion fct
*
* Revision 1.7  1996/06/26  22:36:21  epattiso
* added CU_CStringToNewPString and CU_PStringToNewCString
*
* Revision 1.6  1996/05/30  20:34:16  jehresma
* Rearranged position of EXPORT_PREFIX
*
* Revision 1.5  1996/03/13  16:04:25  jehresma
* win32 round 1
*
* Revision 1.4  1996/02/19  20:17:45  sdeibel
* Removed "const" from CU_TXUseCString since that was technically
* incorrect; this function is problematic anyway...
*
* Revision 1.3  1996/02/19  03:15:28  jehresma
* Added CU_TXCopy
*
* Revision 1.2  1996/02/17  19:31:04  jont
* Added C++ declaration bracketing
*
 * Revision 1.1  1996/01/23  23:31:58  sdeibel
 * Initial revision
 *
* 
*/

#ifndef CU_TEXT_H_ALREADY_INCLUDED
#define CU_TEXT_H_ALREADY_INCLUDED


/**************************************************************************/
/* INCLUDES */

#ifdef dos
#include "CUBasics.h"
#include "CUFail.h"
#else
#include "CU_Basics.h"
#include "CU_Failure.h"
#include "CU_Sequence.h"
#endif /* dos */

#include <string.h>


/**************************************************************************/
/* CONSTANTS */


/**************************************************************************/
/* TYPES */

typedef struct t_CUt_Text {
  char * text;
  tUInt4 length;
  tUInt4 buffer_size;
  tUInt4 block_size;
  tBoolean borrowed;
} CUt_Text;

/* Sequence */
typedef CU_DEFINE_SEQ(CUt_Text) CUt_TextSeq;


/**************************************************************************/
/* PROTOTYPES */
#ifdef __cplusplus
extern "C" {
#endif

/* Text conversion utilities */

EXPORT_PREFIX void CU_PStringToCString (char * the_string);
EXPORT_PREFIX char * CU_PStringToNewCString (const char * p_string);
EXPORT_PREFIX void CU_CStringToPString (char * the_string);
EXPORT_PREFIX char * CU_CStringToNewPString (const char * c_string);

EXPORT_PREFIX tInt4 CU_StrCaseCmp (const char *s1, const char *s2);
EXPORT_PREFIX tInt4 CU_StrNCaseCmp (const char *s1, const char *s2, tUInt4 max_len);
EXPORT_PREFIX char * CU_StrDup (const char* src);
EXPORT_PREFIX void CU_StrDeleteChar(char* string, tUInt4 start, tUInt4 length);
EXPORT_PREFIX void CU_StripString(char *str);

/* CU Text facility */

EXPORT_PREFIX CUt_Text * CU_TXCreate();
EXPORT_PREFIX void CU_TXInit(CUt_Text *str);
EXPORT_PREFIX void CU_TXClear(CUt_Text *str);
EXPORT_PREFIX void CU_TXDestroy(CUt_Text *str);

EXPORT_PREFIX void CU_TXSetBlockSize(CUt_Text *str, tUInt4 block_size);

EXPORT_PREFIX tError CU_TXSetText
      (CUt_Text *str, const char *text, tUInt4 len);
EXPORT_PREFIX void CU_TXSetTextPtr
      (CUt_Text *str, char *text, tUInt4 len, tUInt4 buffer);
EXPORT_PREFIX void CU_TXUseCString(CUt_Text *str, char *text);
EXPORT_PREFIX void CU_TXUseTextPtr(CUt_Text *str, char *text, tUInt4 len);

EXPORT_PREFIX tError CU_TXAppendChar(CUt_Text *str, const char the_char);
EXPORT_PREFIX tError CU_TXAppendText
      (CUt_Text *str, const char *text, tUInt4 len);
EXPORT_PREFIX tError CU_TXAppendCString(CUt_Text *str, const char *cstr);
EXPORT_PREFIX tError CU_TXAppendCUText(CUt_Text *str, const CUt_Text *dstr);

EXPORT_PREFIX tUInt4 CU_TXGetLength(const CUt_Text *str);

EXPORT_PREFIX void CU_TXGetText(CUt_Text *str, char *text, tUInt4 *len);
EXPORT_PREFIX void CU_TXGetTextPtr(CUt_Text *str, char **text, tUInt4 *len);

EXPORT_PREFIX void CU_TXGetCString
      (const CUt_Text *str, char *cstr, tUInt1 max_len);
EXPORT_PREFIX const char * CU_TXGetCStringPtr(const CUt_Text *str);

EXPORT_PREFIX tError CU_TXTextToCString(const CUt_Text *text, char **str);

EXPORT_PREFIX void CU_TXGetPString
      (const CUt_Text *str, char *pstr, tUInt1 max_len);

EXPORT_PREFIX tBoolean CU_TXCompareText
      (const CUt_Text *txt1, const char *text, tUInt4 len);
EXPORT_PREFIX tBoolean CU_TXCompareCString
      (const CUt_Text *txt1, const char *str);
EXPORT_PREFIX tBoolean CU_TXComparePString
      (const CUt_Text *txt1, const char *pstr);
EXPORT_PREFIX tBoolean CU_TXCompareCUText
      (const CUt_Text *txt1, const CUt_Text *txt2);

EXPORT_PREFIX tError CU_TXTruncate(CUt_Text *text, tUInt4 len);

EXPORT_PREFIX tError CU_TXExpandBuffer(CUt_Text *str, tUInt4 new_size);
EXPORT_PREFIX tError
CU_TXDeleteRange(CUt_Text* text, tUInt4 start_pos, tUInt4 length);
tError
CU_TXInsertText(CUt_Text* text, tUInt4 pos, const char* str, tUInt4 str_len);

EXPORT_PREFIX tError CU_TXCopy(CUt_Text *dest, const CUt_Text *src);

/* Sequence utilities */

EXPORT_PREFIX void CU_TXClearSequence(CUt_TextSeq* seq);

#ifdef __cplusplus
} /* extern "C" */
#endif 

#endif /* CU_TEXT_H_ALREADY_INCLUDED */

