/***************************************************************************
* 
* Copyright (c) 1998, Decision Systems Group
*
* Permission is hereby granted, free of charge, to any person obtaining
* a copy of this software and associated documentation files (the
* "Software"), to deal in the Software without restriction, including
* without limitation the rights to use, copy, modify, merge, publish,
* distribute, sublicense, and/or sell copies of the Software, and to
* permit persons to whom the Software is furnished to do so, subject to
* the following conditions:
*
* The above copyright notice and this permission notice shall be
* included in all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
* 
* ==========================================================================
* 
* FILE: CU_Thread.h
* 
* AUTHOR: 
*
* Stephan R.A. Deibel
* 
* CREATION DATE: 
* 
* 12/15/93
*
* VERSION: 
* 
* 8/4/94
*
* DESCRIPTION: 
* 
* A cross-platform threads package for multi-stack non-preemptive thread
* multitasking on a variety of hardware and compiler platforms.
* 
* NOTES: 
* 
* 
* MODIFICATIONS: 
* --------------------------------------------------------------------------
* Date     Name      Description of modification
* --------------------------------------------------------------------------
* 
* $Log: CU_Thread.h,v $
* Revision 1.2  1998/07/15 05:20:16  sdeibel
* Changed all copyright notices to match MIT license release plus misc
* prep for 0.8.3 release
*
 * Revision 1.1.1.1  1997/10/24  16:48:44  sdeibel
 * Converting to CVS
 *
* Revision 1.19  1996/07/02 14:18:32  sdeibel
* Fixed definition of PREEMPTIVEMULTI flag!
*
* Revision 1.18  1996/05/30  20:34:16  jehresma
* Rearranged position of EXPORT_PREFIX
*
* Revision 1.17  1996/04/12  20:51:55  jehresma
* Win32 threads implementation
*
* Revision 1.16  1996/02/12  05:18:00  jehresma
*  Linux modifications
*
 * Revision 1.15  1996/01/18  20:57:41  sdeibel
 * Added use of CUk_NOTHREADS macro, so header file is nullified when threads
 * are not in use
 *
 * Revision 1.14  1996/01/10  19:59:14  sdeibel
 * Port back to Mac plus partial port to PPC / Mac ThreadManager
 * (to be continued)
 *
*
* 12/15/93 Stephan   Created file / initial implementation
* 8/4/94   Stephan   Rewrote based on some code from Michael Hecht
*                    and some creative hacking to swap out stacks
*                    rather than copying them on each context switch.
*                    Also applied DSG naming and commenting conventions.
* 8/8/94   Stephan   Ported to SunOS 4.x.
* 8/10/94  Stephan   Augmented blocking facility.
* 9/21/94  Stephan   Added support for smart process yielding on non-
*                    preemptive systems.  Also some formatting/cosmetics.
* 7/18/95  Stephan   Ported to QuickThreads, so it will run on HP/UX 9.0.5
*                    among other systems.
* 
*/

#ifndef CUk_NOTHREADS

#ifndef CU_THREAD_H_ALREADY_INCLUDED
#define CU_THREAD_H_ALREADY_INCLUDED

#ifdef dos 
#include "CUBasics.h"
#include "CUFail.h"
#else
#include "CU_Basics.h"
#include "CU_Failure.h"
#endif /* dos */

#ifdef __cplusplus
extern "C" {
#endif


/**************************************************************************/
/* ENVIRONMENT CHECKS */

/* Check that one and only one of the implementation modes is defined */
/* and that the hardware is acceptible for that */

#ifdef SETJMPTHREADS
#if defined(SUNLWP) || defined(QUICKTHREADS) || defined(MACTHREADMANAGER) || defined(WIN32THREADS)
#error Duplication implementation modes set
#elif (!defined(macintosh) && !defined(dos))
#error Unsupported SETJMP threads implementation platform
#endif

#elif defined(SUNLWP)
#if defined(SETJMPTHREADS) || defined(QUICKTHREADS) || defined(MACTHREADMANAGER) || defined(WIN32THREADS)
#error Duplication implementation modes set
#elif !defined(sunos4)
#error Unsupported SUNLWP threads implementation platform
#endif

#elif defined(QUICKTHREADS)
#if defined(SUNLWP) || defined(SETJMPTHREADS) || defined(MACTHREADMANAGER) || defined(WIN32THREADS)
#error Duplication implementation modes set
#elif !defined(hpux9) && !defined(sunos4) && !defined(linux)
#error Untested QuickThreads implementation platform; may work however!
#error You will have to fill in stack min/default size constants in
#error CU_Thread.prv.h and should adjust the Environment checks here
#error as appropriate.
#endif 

#elif defined(MACTHREADMANAGER)
#if defined(SUNLWP) || defined(SETJMPTHREADS) || defined(QUICKTHREADS) || defined(WIN32THREADS)
#error Duplication implementation modes set
#elif !defined(macintosh)
#error Unsupported MACTHREADMANAGER threads implementation platform
#endif 

#elif defined(WIN32THREADS)
#if defined(SUNLWP) || defined(SETJMPTHREADS) || defined(QUICKTHREADS) || defined(MACTHREADMANAGER)
#error Duplication implementation modes set
#elif !defined(WIN32)
#error Unsupported WIN32THREADS threads implementation platform
#endif 

#else
#error Threads implementation type not defined!!!
#endif /* Threading type */

/* Check special Mac configuration requirements */
#ifdef macintosh
#if !defined(THINK_C) && !defined(__MWERKS__)
#error Unsupported Macintosh Compiler environment
#endif
#if (GENERATINGPOWERPC || !defined(mc68000)) && defined(SETJMPTHREADS)
#error SETJMPTHREADS work on mc68000 Macs only; use MACTHREADMANAGER on PPC systems
#endif
#endif /* macintosh */

/* Set PREEMPTIVEMULTI based on system type */
#if defined(MACTHREADMANAGER) || defined(SETJMPTHREADS) || defined(WIN32THREADS)
#ifdef PREEMPTIVEMULTI
#undef PREEMPTIVEMULTI
#endif
#else
#ifndef PREEMPTIVEMULTI
#define PREEMPTIVEMULTI
#endif
#endif


/**************************************************************************/
/* CONSTANTS */

#define CUk_UseDefaultStackSize 0


/**************************************************************************/
/* TYPES */

/*
* Procedure types
*/

typedef EXPORT_PREFIX void (*CUt_ThreadProc) (void *user_data);
typedef tBoolean (*CUt_ThreadBlockProc)(void *user_data, void *params);

/*
* Thread ID type
*/

typedef tUInt4 CUt_ThreadID;

/*
* Thread blocking options
*/

typedef enum {
  CUk_UnBlock = 0L,
  CUk_Delay = 1L,
  CUk_WhileFALSE = 2L,
  CUk_UntilFALSE = 3L,
  CUk_WhileEqual = 4L,
  CUk_UntilEqual = 5L,
  CUk_WhileFctFALSE = 6L,
  CUk_UntilFctFALSE = 7L
} CUt_BlockOpt;

/* 
* Application status types 
*/

typedef enum {
  CUk_InBackground = 0L,
  CUk_InForeground = 1L,
  CUk_Quitting = 2L
} CUt_ApplStatus;


/**************************************************************************/
/* ACCESS MACROS */


/**************************************************************************/
/* OTHER MACROS */


/**************************************************************************/
/* GLOBALS */

/*
* Application status - either in foreground (interacting with user),
* in background, or quitting.
*/

extern CUt_ApplStatus CUg_ApplStatus;


/**************************************************************************/
/* PROTOTYPES */

/* ----------------- The core thread manager package ------------------- */

/* Start/stop tasking */
CUt_ThreadID EXPORT_PREFIX CU_StartTasking(void *data);
void EXPORT_PREFIX CU_StopTasking(void);

/* Create/destroy threads */
CUt_ThreadID EXPORT_PREFIX CU_CreateThread(CUt_ThreadProc proc,
  void *data, size_t stack_size);
void EXPORT_PREFIX CU_DestroyThread(CUt_ThreadID ID);

/* Misc access functions */
CUt_ThreadID EXPORT_PREFIX CU_GetActiveThread();
CUt_ThreadID EXPORT_PREFIX CU_GetMasterThread();
EXPORT_PREFIX void * CU_GetThreadData(CUt_ThreadID ID);
EXPORT_PREFIX void * CU_SetThreadData(CUt_ThreadID ID, void *user_data);

size_t EXPORT_PREFIX CU_GetThreadStackSpace(CUt_ThreadID ID);
tUInt4 EXPORT_PREFIX CU_CountActiveThreads();
/* PROBABLY NOT NEEDED! */
#if 0
tUInt4 EXPORT_PREFIX CU_GuessNextRunTime(tBoolean exclude_current);
#endif

/* Thread delaying / blocking function */
void EXPORT_PREFIX CU_BlockThread(CUt_ThreadID ID, CUt_BlockOpt opt, 
     void *p1, void *p2, void *p3, void *p4, tBoolean local_peer);

/* Cooperative thread time slicing functions */               
void EXPORT_PREFIX CU_ThreadYield(void);
tBoolean EXPORT_PREFIX CU_StartThread(CUt_ThreadID ID);

/* ---------------------- Additional facilities -------------------------- */

/* Notify this package of change in application status */
void EXPORT_PREFIX CU_ApplStatusChanged(CUt_ApplStatus status);

/* --------------- For cooperative multitasking systems ------------------ */

/* Thread procedure for smart yielding of time to other processes */
/* in a cooperative multitasking environment.  Does nothing in */
/* a transparent manner on preemptive systems */
void EXPORT_PREFIX CU_ProcessYielder(void *data);


#ifdef __cplusplus
} /* extern "C" */
#endif 

#endif /* !CU_THREAD_H_ALREADY_INCLUDED */

#endif /* !CUk_NOTHREADS */
