/***************************************************************************
* 
* Copyright (c) 1998, Decision Systems Group
*
* Permission is hereby granted, free of charge, to any person obtaining
* a copy of this software and associated documentation files (the
* "Software"), to deal in the Software without restriction, including
* without limitation the rights to use, copy, modify, merge, publish,
* distribute, sublicense, and/or sell copies of the Software, and to
* permit persons to whom the Software is furnished to do so, subject to
* the following conditions:
*
* The above copyright notice and this permission notice shall be
* included in all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
* 
* ==========================================================================
* 
* FILE: CU_Time.h
* 
* AUTHOR: 
* 
* Stephan R.A. Deibel
*
* CREATION DATE: 
* 
* 12/16/93
*
* VERSION: 
* 
* 8/8/94
*
* DESCRIPTION: 
* 
* Header for basic cross-platform time-related utilities.
* 
* NOTES: 
* 
* 
* MODIFICATIONS: 
* --------------------------------------------------------------------------
* Date     Name      Description of modification
* --------------------------------------------------------------------------
* 
* 12/16/93 Stephan   Created file / initial implementation
* 8/8/94   Stephan   Added include of Basics.h for types in here.
*
*/

#ifndef CU_TIME_H_ALREADY_INCLUDED
#define CU_TIME_H_ALREADY_INCLUDED

#ifdef dos
#include "CUBasics.h"
#else
#include "CU_Basics.h"
#endif /* dos */

#ifdef macintosh
#include <Timer.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif


/**************************************************************************/
/* CONSTANTS */

/* Maximum delay in microseconds that the microsecond timing facility */
/* can currently handle */
#define CUk_MaxUSDelay 100 * 1000 * 1000  /* 10 minutes */


/**************************************************************************/
/* TYPES */

/* Types for the microsecond timing facility */
#ifdef macintosh
typedef struct {
  tUInt4 timer_ID;
  tBoolean running;
  struct TMTask tm_task;
  tUInt4 overhead;
} CUt_USTimer;
#endif
#ifdef unix
typedef struct {
  tUInt4 timer_ID;
  tBoolean running;
  /* INCOMPLETE */
} CUt_USTimer;
#endif
#ifdef MS_WIN
typedef struct {
  tUInt4 timer_ID;
  tBoolean running;
  /* INCOMPLETE */
} CUt_USTimer;
#endif


/* Time value is an unsigned 8 byte quantity which measures time in 
   100 nanosecond units.  Absolute time is measured from 15 Oct 1582 00:00:00.
   The upper bound for an absolute date in this format is ~30,000 AD.
   This representation conforms to the X/Open DCE Time Service and the 
   CORBA time service */
#ifdef MS_WIN // Avoid VC++ compiler bug
typedef tUInt4 CUt_TimeValue;
#else
typedef tUInt8 CUt_TimeValue;
#endif


/* A structure to separate time quantities */
typedef struct CUt_AbsDate {
  tUInt2 year;
  tUInt1 month;
  tUInt1 day;
  tUInt1 hour;
  tUInt1 minute;
  tUInt1 second;
  tUInt4 nanosecond;  
  tInt2 minute_offset;  //SJW: for local time zones, measured in minutes from UTC (?)
} CUt_AbsDate;


/**************************************************************************/
/* ACCESS MACROS */


/**************************************************************************/
/* OTHER MACROS */


/**************************************************************************/
/* GLOBALS */


/**************************************************************************/
/* PROTOTYPES */

EXPORT_PREFIX tBoolean CU_InitTicks(void);
EXPORT_PREFIX tUInt4 CU_CurrentTicks(void);

EXPORT_PREFIX CUt_USTimer * CU_CreateUSTimer(void);
EXPORT_PREFIX void CU_StartUSTimer(CUt_USTimer *timer);
EXPORT_PREFIX tUInt4 CU_CheckUSTimer(CUt_USTimer *timer);
EXPORT_PREFIX tUInt4 CU_StopUSTimer(CUt_USTimer *timer);
EXPORT_PREFIX void CU_DestroyUSTimer(CUt_USTimer *timer);

CUt_TimeValue CU_CurrentUtcTime();
CUt_TimeValue CU_AbsDateToTime(const CUt_AbsDate* vtime);
CUt_AbsDate CU_TimeToAbsDate(CUt_TimeValue time, tInt2 minute_offset);
tInt2 CU_GetCurrentMinuteOffset();
void CU_AddYearsToTimeValue(CUt_TimeValue* tv, tInt4 years);
void CU_AddDaysToTimeValue(CUt_TimeValue* tv, tInt4 days);
void CU_AddSecondsToTimeValue(CUt_TimeValue* tv, tInt4 seconds);
void CU_AddNanoSecondsToTimeValue(CUt_TimeValue* tv, tInt8 nanoseconds);
tUInt4 CU_GetYearsOfTimeValue(CUt_TimeValue tv);
tUInt4 CU_GetDaysOfTimeValue(CUt_TimeValue tv);
tUInt4 CU_GetSecondsOfTimeValue(CUt_TimeValue tv);
tUInt8 CU_GetNanoSecondsOfTimeValue(CUt_TimeValue tv);


#ifdef __cplusplus
} /* extern "C" */
#endif 

#endif /* CU_BASICS_H_ALREADY_INCLUDED */

