/***************************************************************************
* 
* Copyright (c) 1998, Decision Systems Group
*
* Permission is hereby granted, free of charge, to any person obtaining
* a copy of this software and associated documentation files (the
* "Software"), to deal in the Software without restriction, including
* without limitation the rights to use, copy, modify, merge, publish,
* distribute, sublicense, and/or sell copies of the Software, and to
* permit persons to whom the Software is furnished to do so, subject to
* the following conditions:
*
* The above copyright notice and this permission notice shall be
* included in all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
* 
* ==========================================================================
* 
* FILE: CU_Tree.h
* 
* AUTHOR: 
*
* Stephan R.A Deibel
* YongJoon Lee
* Karen Dintino
* 
* CREATION DATE:
*
* June 1994 
* 
* DESCRIPTION: 
*
* Header file for a generic tree structure.
* 
* NOTES: 
* 
* 
* MODIFICATIONS: 
* --------------------------------------------------------------------------
* Date     Name      Description of modification
* --------------------------------------------------------------------------
*
* $Log: CU_Tree.h,v $
* Revision 1.3  1998/12/01 23:18:50  bsloan
* Port to MetroWerks CodeWarrior Pro4 for Windows
*
* Revision 1.2  1998/07/15 05:20:17  sdeibel
* Changed all copyright notices to match MIT license release plus misc
* prep for 0.8.3 release
*
 * Revision 1.1.1.1  1997/10/24  16:48:45  sdeibel
 * Converting to CVS
 *
* Revision 1.13  1997/05/05 20:17:51  sdeibel
* Misc 2
*
* Revision 1.12  1997/05/05  20:17:22  sdeibel
* Misc
*
* Revision 1.11  1997/02/18  21:39:43  jrequena
* more sibling support
*
* Revision 1.10  1997/01/15  21:08:57  jrequena
* Added sibling support .
*
* Revision 1.9  1996/06/03  03:54:04  sdeibel
* Clean up / added functions
*
* 02/14/95 YJLee    Changed FAR PASCAL to EXPORT_PREFIX
*                   Fixed TraverseInorder to TraverseInOrder.
* 11/15/94 Karen    Insert the MS Win prefix (FAR PASCAL) for remote calling. 
* 
*/

#ifndef CU_TREE_H_ALREADY_INCLUDED
#define CU_TREE_H_ALREADY_INCLUDED

#ifdef __cplusplus
extern "C" {
#endif


/**************************************************************************/
/* TYPES */

/* Tree node data type */
typedef struct CUt_TreeNode {
  struct CUt_TreeNode *parent;
  struct CUt_TreeNode *first_child;
  struct CUt_TreeNode *next_sib;
  struct CUt_TreeNode *prev_sib;
  void *data;
} CUt_TreeNode;

/* Traversal function:  Returns TRUE to halt traversal; FALSE to continue */
typedef tBoolean (*CUt_TraversalFn)(CUt_TreeNode *, void *);

/* Guide function:  Returns child to traverse to; NULL to halt */
typedef CUt_TreeNode *(*CUt_GuideFn)(CUt_TreeNode *, void *);


/**************************************************************************/
/* PROTOTYPES */

/* Node creation / destruction */
EXPORT_PREFIX CUt_TreeNode *CU_TN_Create();
EXPORT_PREFIX void CU_TN_Destroy(CUt_TreeNode *node);

/* Tree construction / destruction */
EXPORT_PREFIX void CU_Tree_AppendChild(CUt_TreeNode *new_node, 
                                       CUt_TreeNode *parent);
EXPORT_PREFIX void CU_Tree_AddSibling(CUt_TreeNode *new_node, 
                                       CUt_TreeNode *sibling);
EXPORT_PREFIX void CU_Tree_PrependChild(CUt_TreeNode *new_node, 
                                        CUt_TreeNode *parent);
EXPORT_PREFIX void CU_Tree_MoveRight(CUt_TreeNode *node);
EXPORT_PREFIX void CU_Tree_MoveLeft(CUt_TreeNode *node);
EXPORT_PREFIX void CU_Tree_InsertChildAfter(CUt_TreeNode *new_node, 
                                            CUt_TreeNode *prev_sibling);
EXPORT_PREFIX void CU_Tree_RemoveNode(CUt_TreeNode *the_node);

/* Inspection */
EXPORT_PREFIX tBoolean CU_Tree_IsDescendentOf(CUt_TreeNode *the_node, 
                                              CUt_TreeNode *ancestor);
EXPORT_PREFIX tBoolean CU_Tree_IsSiblingOf(CUt_TreeNode *the_node, 
																						CUt_TreeNode *root_node);                                              
EXPORT_PREFIX tUInt4 CU_Tree_GetNodeDepth(CUt_TreeNode *the_node);

/* Traversal functions */
EXPORT_PREFIX CUt_TreeNode *CU_Tree_TraverseChildren(CUt_TreeNode *parent,
                                                     CUt_TraversalFn Action, 
                                                     void *data);
EXPORT_PREFIX CUt_TreeNode *CU_Tree_TraverseSiblings(CUt_TreeNode *node, 
						     CUt_TraversalFn Action, 
						     void* data);
EXPORT_PREFIX CUt_TreeNode *CU_Tree_TraverseInOrder(CUt_TreeNode *root, 
                                                    CUt_TraversalFn Action, 
                                                    void *data);
EXPORT_PREFIX CUt_TreeNode *CU_Tree_TraverseGuided(CUt_TreeNode *root, 
                                                   CUt_GuideFn Guide, 
                                                   void *data);
EXPORT_PREFIX CUt_TreeNode *CU_Tree_NextInOrderNode(CUt_TreeNode *node);
EXPORT_PREFIX CUt_TreeNode *CU_Tree_PrevInOrderNode(CUt_TreeNode *node);
EXPORT_PREFIX CUt_TreeNode* CU_Tree_LastSibling(CUt_TreeNode *node);
EXPORT_PREFIX CUt_TreeNode* CU_Tree_LastInSubTree(CUt_TreeNode *node);


#ifdef __cplusplus
} /* extern "C" */
#endif 

#endif /* CU_TREE_H_ALREADY_INCLUDED */
